const svg = d3.select('svg');

const height = +svg.attr('height');
const width = +svg.attr('width');



//---------------------------------------------------------------

//RENDER FUNCTION!!!!
const render = function(data, x_value_name, y_value_name){
 
  
  //Configuring margins
  const margin = {top:20, bottom:100, right:20, left:120};
  const inner_width = width - margin.left - margin.right;
  const inner_height = height - margin.top - margin.bottom;
  
  //Creating a group that will be the chart itself
  const g = svg.append('g')
  							.attr('transform', `translate (${margin.left} , ${margin.top})`);
  
  //Setting x and y value variables for flexibility
  const x_value = function(d){ return d[x_value_name]};
  const y_value = function(d){ return d[y_value_name]};
  
  //Configuring axes scale
  const x_scale = d3.scaleLinear()
    .domain([0, d3.max(data, x_value)])
  	.range([0,inner_width]);
  
  const y_scale = d3.scaleBand()
  	.domain(data.map(y_value))	
  	.range([0,inner_height])
  	.padding(0.1);
  
	//Configuring axes
  //--Creating axes group
  const axes = g.append('g').attr("class", "axes").attr("fill", "red");
  
  //--Function to format numbers in ticks
  const format_ticks_number = function(number) {
    return d3.format(".3s")(number)
      	.replace("G", "B")
  };
  
  //--Removing first value (0) from xscale values
  const x_ticks_values = x_scale.ticks().slice(1);
  
  //--Saving instances of axis creating functions
  const create_y_axis = d3.axisLeft(y_scale);

  const create_x_axis = d3.axisBottom(x_scale)
  									.tickFormat(format_ticks_number)
  									.tickSizeOuter(0)
  									.tickSize(-inner_height)
  									.tickValues(x_ticks_values);
  
  
  //--Creating axes
  const y_axis = axes.append('g')
  	.attr("class", "y_axis")
    .call(create_y_axis); //calling in function y_scale and passing it to selection
  
  
  const x_axis = axes.append('g')
    .attr("class", "x_axis")
    .call(create_x_axis)
      .attr('transform', `translate (0 , ${inner_height})`); //moving x axis to bottom


  //--Removing unwanted elements from axes
  y_axis.selectAll('.domain, .tick line') //selects both ".domain" path element and "line" element in "tick(class)" 
  			.remove();
  
  x_axis.selectAll('.domain') // Removing ".domain"" 
  			.remove();
  
  //--Creating x axis label
  const x_label = x_axis.append('text')
				.text(x_value_name.toUpperCase())
          .attr('class','x_label')
          .attr('y', '50')
  				.attr('x', inner_width/2);
  
  const foreign = g.append("foreignObject")
  .attr("class", "svg-tooltip");

  const tooltip = foreign.append('xhtml:div')
    //.append("div")
    .attr("class", "tooltip")
  	.style("display","table-cell")	
  	.style("text-align", "center")
  	.style("vertical-align", "middle");
  
  
  const tooltip_text = tooltip.append("p")
    .attr("id", "tooltip-text")  	;
  
  const tooltip_key = tooltip_text.append("span")
  	.attr("id", "tooltip-key");
  
  const tooltip_value = tooltip_text.append("span")
  	.attr("id", "tooltip-value");





  
  //--Creating Sum Box
  
  const sum_obj = d3.select('g').append('g')
    .style("opacity", "0")
  const sum_box = sum_obj.append('rect')
    .attr('id','sum_rect')
  const sum_text = sum_obj.append("text")
  

	//Data joined and mapped. On the same selection, mouse behaviour will be defined
  g.selectAll("rect:not(#sum_rect)").data(data)
  	.enter()
    .append('rect')
  		.attr('width', function(d) {return x_scale( x_value(d) ) } )
  		.attr('height', y_scale.bandwidth)
  		.attr('y', function(d) {return y_scale( y_value(d) ) } )
   	
  	//--Mouseover behaviour defined here
    .on('mouseover', function (d) {
    
    	//First define text
      tooltip_key.text(y_value(d) + ": ")
    		.style("font-weight", "bold")
    	tooltip_value.text(d3.format(",d")(x_value(d)) + " inh.")      
    
      
    	//Set dimension variables
    	var tooltip_height = tooltip.node().getBoundingClientRect().height;
      var tooltip_width = tooltip.node().textContent.length * 10;
      var mouse_x = d3.mouse(d3.event.target)[0];
      var mouse_y = d3.mouse(d3.event.target)[1];
      var xPosition = function() {return d3.event.pageX- 100; };
      var yPosition =function() {return d3.event.pageY;};
    
    
    	//Set foreign object dimensions
    	foreign
        .attr("x", xPosition())
        .attr("y", yPosition())
        .attr("width", tooltip_width)
        .attr("height", 100);

      //Bring foreign object upfront
    	foreign.raise()

      //Make tooltip appear
    	tooltip.transition()
        .duration('200')
        .style("opacity", "0.7")

       
  
  })
  	.on('mouseout', function (d) {

    //Make tooltip disappear
    tooltip.transition()
      .duration('200').style("opacity", "0")		
    
  })
    
  //-- Display box with sum of selected rects
	.on('click', function(d){
  	
    const clicked = d3.select(this)
    
    //----saving colors by meaning
    const	selected_color = 'rgb(255, 64, 112)'
    const	unselected_color = 'rgb(50, 230, 155)'	
    	

    //----initializing empty objects for later
    let sum_dict = {}
    let count_selected = 0

    //----getting a count of selected rects before click
    //----getting a dictionary of selected rects before click
    g.selectAll('rect:not(#sum_rect)').each(function(d,i) { 

      let element_rgb = d3.select(this).style('fill');

      if (element_rgb == selected_color) {
        count_selected += 1;
        sum_dict[y_value(d)] = x_value(d);
      };
    });
    
    
    //----changing color of clicked rect
    //----adding or removing clicked rects from dictionary
    if (clicked.style('fill') == unselected_color) {
      clicked.transition()
    		.duration('50')
    		.style('fill', selected_color);
      
      sum_dict[y_value(d)] = x_value(d);
    } 
    else {
      clicked.transition()
    		.duration('50')
    		.style('fill', unselected_color);
      
      delete sum_dict[y_value(d)]};
    
    
		//----Closing sum box when last selected rect is clicked
    //----In all other cases sum box is updated 
    
    if (count_selected === 1 && clicked.style('fill') == selected_color){
        sum_obj.transition()
        .duration('500')
        .style('opacity', '0');
     } 
  	else {        
        const sumValues = obj => Object.values(obj).reduce((a, b) => a + b);
      
        const sum_box_height = 50
        const sum_box_width = 150
        const box_left_margin = 10

        
        sum_box.attr('x', '0')
        .attr('y', inner_height/2 -(sum_box_height/2)+5)
        .attr('width', sum_box_width)
        .attr('height', sum_box_height)
      	.style('fill', "rgb(68,68,68)")
        .style("opacity", "0.6");
        
        
        sum_text.attr("x", box_left_margin)
        .attr("y", inner_height/2)
        .attr('font-size','0.8em')
      	.attr('fill', 'white')
        .text('SUM SELECTION:')
        .append('tspan')
        .text(d3.format(",d")(sumValues(sum_dict)))
        .attr("dy", "1.3em")
        .attr('x',box_left_margin)
      	.style('font-size', '1.2em');
        
        
        

        sum_obj.attr("transform", `translate(${inner_width-sum_box_width}, 0)`)
      	.transition()
        .duration('500')
        .style('opacity', '0.8')


      
      
      }
    

  
    }
  );
};


//---------------------------------------------------------------


//DATA THAT IS PASSED TO RENDER FUNCTION!!!
const pop_data = d3.csv("data.csv").then(
  function(data){
    data.forEach(
      function(value){
      	value.population = +value.population  * 1000;
      });
    
    //execute render function
    render(data, 'population', 'country');
    }
);





